"""Visualization for base diagrams.

This module contain methods for base diagrams visualization.
"""

import numpy as np
import matplotlib.pyplot as plt

from itertools import cycle
from .colors import Colors

FIGSIZE = (16, 9)


def visualize_scatter(
    points: np.ndarray,
    labels: np.ndarray,
    colors: list[Colors] | None = None,
    axis: plt.Axes | None = None,
) -> None:
    """
    Visualize scatter with labels.

    Args:
        points: Points.
        labels: Labels.
        colors: Colors for labels.
        axis: Axis.
    """

    # TODO chech sizes of input data

    if colors is None:
        colors = list(Colors)

    colors_cycle = cycle(colors)
    labels_unique = np.unique(labels)

    if axis is None:
        _, axis = plt.subplots(figsize=FIGSIZE)

    for label in labels_unique:
        label_mask = labels == label
        axis.scatter(*points[label_mask].T, color=next(colors_cycle))

    axis.grid(True)


def visualize_violin(
    data: np.ndarray,
    axis: plt.Axes
) -> None:
    """
    Visualize violin.

    Args:
        data: Data.
        axis: Axis.
    """

    axis.violinplot(
        data,
        vert=False,
        showmedians=True,
    )
    axis.set_yticks([])


def visualize_hist(
    data: np.ndarray,
    axis: plt.Axes
) -> None:
    """
    Visualize hist.

    Args:
        data: Data.
        axis: Axis.
    """

    axis.hist(
        data,
        bins=100,
        color="cornflowerblue",
        density=True,
        alpha=0.5,
    )


def visualize_boxplot(
    data: np.ndarray,
    axis: plt.Axes
) -> None:
    """
    Visualize boxplot.

    Args:
        data: Data.
        axis: Axis.
    """
        
    axis.boxplot(
        data,
        vert=False,
        patch_artist=True,
        boxprops=dict(facecolor="blue"),
        medianprops=dict(color="black"),
    )



"""Visualization of regressor.

This module contain methods for regressor visualization.
"""

import matplotlib.pyplot as plt
import numpy as np

from ._utils import _save_fig

FIGSIZE = (16, 9)


def visualize_regressor(
    abscissa: np.ndarray,
    ordinates: np.ndarray,
    predictions: np.ndarray,
    error: np.ndarray,
    path_to_save: str = ""
) -> None:
    """
    Visualize regressor prediction.

    Args:
        abscessa: Abscissa of data.
        ordinates: Ordinates of data.
        prediction: Regressor prediction.
        error: Prediction error.
        path_to_save: Path to save visualization.
            Defaults to "".

    Examples:
        >>> import numpy as np
        >>> from solidipy_mipt import train_test_split
        >>> X = np.array([[1, 2], [3, 4], [5, 6], [7, 8]])
        >>> y = np.array([0, 1, 0, 1])
        >>> X_train, X_test, y_train, y_test = train_test_split(X, y, train_ratio=0.6, shuffle=True)
    """

    # TODO check sizes of input data

    _, axis = plt.subplots(figsize=FIGSIZE)
    axis.scatter(abscissa, ordinates, label='source', c="royalblue", s=1)
    axis.plot(abscissa, predictions, label='prediction', c="steelblue")
    axis.plot(abscissa, predictions - error, label='error', linestyle='--', c="red")
    axis.plot(abscissa, predictions + error, linestyle='--', c="red")
    axis.set_xlim(min(abscissa), max(abscissa))
    axis.legend()

    if (path_to_save):
        _save_fig(path_to_save)



from .distribution import visualize_distribution, DiagramType
from .regression import visualize_regressor
from .comparison import visualize_comparison
from .base_diagrams import visualize_scatter, visualize_violin, visualize_hist, visualize_boxplot


__all__ = [
    "colors",
    "DiagramType",
    "visualize_distribution",
    "visualize_regressor",
    "visualize_comparison",
    "visualize_scatter",
    "visualize_violin",
    "visualize_hist",
    "visualize_boxplot"
]



"""Visualization for distribution.

This module contain methods for distribution visualization.
"""

import matplotlib.pyplot as plt
import numpy as np

from enum import IntEnum
from .base_diagrams import (
    visualize_scatter,
    visualize_violin,
    visualize_hist,
    visualize_boxplot
)
from ._utils import _save_fig


class DiagramType(IntEnum):
    VIOLIN = 1
    HIST = 2
    BOXPLOT = 3


plt.style.use("ggplot")
FIGSIZE = (16, 9)
SPACE = 0.2


def visualize_distribution(
    data: np.ndarray,
    diagram_type: DiagramType,
    path_to_save: str = "",
) -> None:
    """
    Visualize distribution. If distribution is 2D visualize 2 histograms,
      which provides additional information about the distribution of data along the axes.

    Args:
        data: The input data for distribution.
        diagram_type: The diagram type.
        path_to_save: Path to save visualization.
            Defaults to "".

    Examples:
        >>> import numpy as np
        >>> from solidipy_mipt.visualization import visualize_distribution
        >>> absc = np.random.normal(size=1000)
        >>> ordi = np.random.normal(size=1000)
        >>> data = np.column_stack([absc, ordi])
        >>> data = np.random.normal(size=1000)
        >>> visualize_distribution(
        ...     data=data,
        ...     diagram_type=Diagram_type.HIST,
        ...     path_to_save="TEST.png"
        ... )
    """

    if (len(data.shape) == 1):
        _visualize_distribution_1d(data, diagram_type, path_to_save)
    elif (len(data.shape) == 2 and data.shape[1] == 2):
        _visualize_distribution_2d(data, path_to_save)


def _visualize_distribution_1d(
    data: np.ndarray,
    diagram_type: DiagramType,
    path_to_save: str = "",
) -> None:
    """
    Visualize distribution.

    Args:
        data: Data for distribution.
        diagram_type: The diagram type.
        path_to_save: Path to save visualization.
    """

    _, axis = plt.subplots(figsize=FIGSIZE)

    match(diagram_type):
        case DiagramType.VIOLIN:
            visualize_violin(data, axis)
        case DiagramType.HIST:
            visualize_hist(data, axis)
        case DiagramType.BOXPLOT:
            visualize_boxplot(data, axis)
    if (path_to_save):
        _save_fig(path_to_save)


def _visualize_distribution_2d(
    data: np.ndarray,
    path_to_save: str = "",
) -> None:
    """
    Visualize distribution and 2 histograms,
      which provides additional information about the distribution of data along the axes.

    Args:
        data: Data for distribution.
        path_to_save: Path to save visualization.
            Defaults to "".
    """

    figure = plt.figure(figsize=(8, 8))
    grid = plt.GridSpec(4, 4, wspace=SPACE, hspace=SPACE)

    axis_scatter = figure.add_subplot(grid[:-1, 1:])
    axis_hist_vert = figure.add_subplot(
        grid[:-1, 0],
        sharey=axis_scatter,
    )
    axis_hist_hor = figure.add_subplot(
        grid[-1, 1:],
        sharex=axis_scatter,
    )

    abscissa, ordinates = data[:, 0], data[:, 1]
    axis_scatter.scatter(
        abscissa,
        ordinates,
        color="cornflowerblue",
        alpha=0.5
    )
    axis_hist_hor.hist(
        abscissa,
        bins=50,
        density=True,
        color="cornflowerblue",
        alpha=0.5,
    )
    axis_hist_vert.hist(
        ordinates,
        bins=50,
        density=True,
        orientation="horizontal",
        color="cornflowerblue",
        alpha=0.5,
    )

    axis_hist_hor.invert_yaxis()
    axis_hist_vert.invert_xaxis()

    if (path_to_save):
        _save_fig(path_to_save)



"""Colors.

This module contain Colors enum.
"""

from enum import StrEnum


class Colors(StrEnum):
    BLUE = "b"
    RED = "r"
    GREEN = "g"
    CEAN = "c"
    MAGENTA = "m"
    YELLOW = "y"
    BLACK = "k"
    ROYALBLUE = "royalblue"
    STEELBLUE = "steelblue"
    DIMGRAY = "dimgray"
    CORNFLOWERBLUE = "cornflowerblue"
